const { pool } = require("../config/database/database.config");

/*
Add degree information into the database. The method will return 
the id of the newly inserted degree, or null if nothing was added.
*/
async function addDegree(degree, peopleId) {
  // get all the parts of the degree
  let {
    degreeType,
    degreeDepartment,
    degreeCollege,
    degreeYear,
    degreeDescription,
  } = degree;

  console.log("adding degree...");

  try {
    // Use the helper function to determine the degreeTypeId
    let degreeTypeId = await _lookupDegreeTypeId(degreeType);

    // query to insert degree info
    const query = `
        INSERT INTO peopleDegree (peopleId, degreeTypeId, degreeDepartment, degreeCollege, degreeYear, degreeDescription)
        VALUES (?,?,?,?,?,?)
    `;

    // insert data into db
    const [results] = await pool.query(query, [
      peopleId,
      degreeTypeId,
      degreeDepartment,
      degreeCollege,
      degreeYear,
      degreeDescription,
    ]);

    console.log("Inserted degree: ", results.insertId);

    const checkQuery = `SELECT * FROM peopleDegree WHERE peopleDegreeId = ?`;
    const [checkResults] = await pool.query(checkQuery, [results.insertId]);

    console.log(checkResults[0]);

    return results.insertId;
  } catch (error) {
    // return null if there is an error
    console.error(error);
    return null;
  }
}

/*
Function to update the information about a specific degree 
in the database. The function will return true if a row
was successfully updated and false otherwise.
*/
async function updateDegree(peopleDegreeId, peopleId, degreeInfo) {
  console.log("updating degree....");

  // get all the parts of the degree
  let {
    degreeType,
    degreeDepartment,
    degreeCollege,
    degreeYear,
    degreeDescription,
  } = degreeInfo;

  try {
    // Use the helper function to determine the degreeTypeId
    let degreeTypeId = await _lookupDegreeTypeId(degreeType);

    // SQL query to update
    const query = `
      UPDATE peopleDegree
      SET
        peopleId = ?,
        degreeTypeId = ?,
        degreeDepartment = ?,
        degreeCollege = ?, 
        degreeYear = ?,
        degreeDescription = ?
      WHERE peopleDegreeId = ?
    `;

    const [results] = await pool.query(query, [
      peopleId,
      degreeTypeId,
      degreeDepartment,
      degreeCollege,
      degreeYear,
      degreeDescription,
      peopleDegreeId,
    ]);

    // Check if any rows were affected
    if (results.affectedRows === 0) {
      console.error("Degree not found. no rows updated");
      return false;
    }

    console.log("degree info updated: ", peopleDegreeId);

    // return true to signify success
    return true;
  } catch (error) {
    console.error("error updating degree: ", error);
    return false;
  }
}

/*
Function to delete a degree by its ID. 
Returns true if the degree was successfully deleted, false otherwise.
*/
async function deleteDegreeById(peopleDegreeId) {
  console.log("Deleting degree with ID: ", peopleDegreeId);

  try {
    const query = `
      DELETE FROM peopleDegree
      WHERE peopleDegreeId = ?
    `;

    const [results] = await pool.query(query, [peopleDegreeId]);

    // Check if any rows were affected
    if (results.affectedRows === 0) {
      console.error("Degree not found. No rows deleted.");
      return false;
    }

    console.log("Degree successfully deleted: ", peopleDegreeId);

    return true;
  } catch (error) {
    console.error("Error deleting degree: ", error);
    return false;
  }
}

/*
Function to find all degree IDs associated with a person. 
Returns an array of peopleDegreeIds or an empty array if no degrees exist.
*/
async function getDegreeIdsForPerson(peopleId) {
  console.log("Finding degree IDs for person with ID: ", peopleId);

  try {
    // get the degrees for a given peopleId
    const query = `
      SELECT peopleDegreeId 
      FROM peopleDegree
      WHERE peopleId = ?
    `;

    const [results] = await pool.query(query, [peopleId]);

    // Extract the degree IDs
    const degreeIds = results.map((row) => row.peopleDegreeId);

    console.log("Found degree IDs: ", degreeIds);

    return degreeIds;
  } catch (error) {
    console.error("Error finding degree IDs: ", error);
    return [];
  }
}

/*
Function to get all degree information associated with a person.
Returns an array of degree objects or an empty array if no degrees exist.
*/
async function getDegreesForPerson(peopleId) {
  console.log("Finding degrees for person with ID: ", peopleId);

  try {
    // get the full degree information for degreees for person with peopleId
    const query = `
      SELECT 
        d.peopleDegreeId, 
        d.degreeDepartment, 
        d.degreeCollege, 
        d.degreeYear, 
        d.degreeDescription, 
        dt.degreeType
      FROM 
        peopleDegree d
      LEFT JOIN 
        degreeTypeLookup dt ON d.degreeTypeId = dt.degreeTypeId
      WHERE 
        d.peopleId = ?;
    `;

    const [results] = await pool.query(query, [peopleId]);

    // Print out each row returned by the query
    console.log("Raw results from query:", results);

    if (results.length === 0) {
      console.log("No degrees found for person with ID:", peopleId);
      return [];
    }

    const degrees = results.map((row) => ({
      peopleDegreeId: row.peopleDegreeId,
      degreeType: row.degreeType,
      degreeDepartment: row.degreeDepartment,
      degreeCollege: row.degreeCollege,
      degreeYear: row.degreeYear,
      degreeDescription: row.degreeDescription,
    }));

    console.log("Found degrees: ", degrees);

    return degrees;
  } catch (error) {
    console.error("Error fetching degrees for person: ", error);
    return [];
  }
}

// Private helper function to look up the degreeTypeId
async function _lookupDegreeTypeId(degreeType) {
  try {
    // return null for now
    return null;
  } catch (error) {
    console.error(`Error looking up degree type: ${degreeType}`, error);
    throw error;
  }
}

module.exports = {
  addDegree,
  updateDegree,
  deleteDegreeById,
  getDegreeIdsForPerson,
  getDegreesForPerson,
  _lookupDegreeTypeId,
};
