const {
  addAddress,
  getAddressesForPerson,
  deleteAddressById,
  updateAddress,
} = require("../repository/addressRepository");
const {
  addContact,
  getContactsForPerson,
  updateContact,
  deleteContactById,
} = require("../repository/contactRepository");
const {
  addDegree,
  getDegreeIdsForPerson,
  deleteDegreeById,
  updateDegree,
  getDegreesForPerson,
} = require("../repository/degreeRepository");
const {
  addPerson,
  searchPeopleByName,
  deletePersonById,
  updatePerson,
  getPersonById,
} = require("../repository/peopleRepository");

/*
Function to handle the logic for creating a person.
*/
async function createPerson(person) {
  // extract the parts of a person
  const { personInfo, degrees, addresses, contacts, involvements } = person;

  console.log("Adding person....");
  // insert the personInfo into people table
  const newPersonID = await addPerson(personInfo);
  console.log("Added person ID: ", newPersonID);

  // insert the degree information into the database
  for (const degree of degrees) {
    const addedDegree = await addDegree(degree, newPersonID);
    console.log("added degree: ", addedDegree);
  }

  // insert the address information into the database
  for (const address of addresses) {
    const addedAddress = await addAddress(address, newPersonID);
    console.log("added address: ", addedAddress);
  }

  // insert the contact information into the database
  for (const contact of contacts) {
    const addedContact = await addContact(contact, newPersonID);
    console.log("added contact: ", addedContact);
  }

  // insert the involvment information into the database
  for (const involvment of involvements) {
    console.log("not adding involvements yet.");
  }

  // TODO: log database change here

  return newPersonID;
}

/*
Function to call repository and get people with a 
gicen first and last name.
*/
async function findByName(firstName, lastName) {
  // call repository function, null will be returned if no people are found
  const peopleFound = await searchPeopleByName(firstName, lastName);
  return peopleFound;
}

/*
Function to delete person by their id.
*/
async function deleteById(peopleId) {
  // delete the person
  const personDeleted = await deletePersonById(peopleId);

  if (personDeleted) {
    return true;
  }

  return false;
}

/*
Function to update all of a person with the given id's information.
*/
async function updatePersonById(personId, person) {
  console.log("Person: ", person);

  const { personInfo, degrees, addresses, contacts, involvements } = person;

  console.log("Updating person...");

  // update personInfo
  const updatedPersonInfo = await updatePerson(personId, personInfo);
  if (updatedPersonInfo == false) {
    return null;
  }

  console.log("updated person info");

  // update degrees
  if (degrees) {
    await updateDegreesForPerson(personId, degrees);
  }

  // update address info
  if (addresses) {
    await updateAddressesForPerson(personId, addresses);
  }

  // update contact info
  if (contacts) {
    await updateContactsForPerson(personId, contacts);
  }

  if (involvements) {
    await updateInvolvementsForPerson(personId, involvements);
  }
  return true;
}

/*
Function to get all the data that is associated with a single 
person.
*/
async function getPersonFullDetails(personId) {
  // get the personal details
  const personalInformation = await getPersonById(personId);

  // get the degree information
  const degreeInformation = await getDegreesForPerson(personId);

  // get the address information
  const addressInformation = await getAddressesForPerson(personId);

  // get the contact information
  const contactInformation = await getContactsForPerson(personId);

  // get the involvement information (none for now)
  const involvmentInformation = [];

  // return all of the data
  return {
    personalInformation,
    degreeInformation,
    addressInformation,
    contactInformation,
    involvmentInformation,
  };
}

/*
Helper function to update the degree information for a person.
*/
async function updateDegreesForPerson(personId, degrees) {
  // Get the person's current degrees
  const currentDegreeIds = await getDegreeIdsForPerson(personId);
  let updatedDegreeIds = [];

  // Add/update the given degrees
  for (const degree of degrees) {
    const peopleDegreeId = degree.peopleDegreeId || null;

    // degree does not previously have a peopleDegreeId -> this means its new (add degree)
    if (!peopleDegreeId) {
      const addedDegree = await addDegree(degree, personId);
      console.log("Added new degree: ", addedDegree);
    } else {
      // previous peopleDegreeId exists, update the degree
      const updatedDegree = await updateDegree(
        peopleDegreeId,
        personId,
        degree
      );
      console.log("Updated degree: ", updatedDegree);
      if (updatedDegree == true) {
        updatedDegreeIds.push(peopleDegreeId);
      }
    }
  }

  // Delete degrees not included in the update
  for (const degreeId of currentDegreeIds) {
    if (!updatedDegreeIds.includes(degreeId)) {
      await deleteDegreeById(degreeId);
      console.log("Deleted degree: ", degreeId);
    }
  }
}

/*
Helper function to update addresses for a person.
*/
async function updateAddressesForPerson(personId, addresses) {
  // Get the current addresses
  const currentAddresses = await getAddressesForPerson(personId);
  let updatedAddressIds = [];

  for (const address of addresses) {
    const addressId = address.addressId || null;

    if (!addressId) {
      const addedAddress = await addAddress(address, personId);
      console.log("Added new address: ", addedAddress);
    } else {
      const updatedAddress = await updateAddress(addressId, personId, address);
      if (updatedAddress != null) {
        console.log("Updated address: ", updatedAddress);
        updatedAddressIds.push(addressId);
      } else {
        console.log("Error updating adddress");
      }
    }
  }

  // Delete addresses not included in the update
  for (const currentAddress of currentAddresses) {
    if (!updatedAddressIds.includes(currentAddress.addressId)) {
      await deleteAddressById(currentAddress.addressId);
      console.log("Deleted address: ", currentAddress.addressId);
    }
  }
}

/*
Helper function to update contacts for a person.
*/
async function updateContactsForPerson(personId, contacts) {
  // Get the current contacts
  const currentContacts = await getContactsForPerson(personId);
  let updatedContactIds = [];

  for (const contact of contacts) {
    const contactId = contact.peopleContactId || null;

    if (!contactId) {
      const addedContact = await addContact(contact, personId);
      console.log("Added new contact: ", addedContact);
    } else {
      const updatedContact = await updateContact(contactId, personId, contact);
      console.log("Updated contact: ", updatedContact);
      if (updatedContact == true) {
        updatedContactIds.push(contactId);
      }
    }
  }

  // Delete contacts not included in the update
  for (const currentContact of currentContacts) {
    if (!updatedContactIds.includes(currentContact.peopleContactId)) {
      // TODO: delete contact by id
      let success = await deleteContactById(currentContact.peopleContactId);
      if (success == true) {
        console.log("Deleted contact: ", currentContact.peopleContactId);
      } else {
        console.log("Error deleting contact: ", currentContact.peopleContactId);
      }
    }
  }
}

async function updateInvolvementsForPerson(personId, involevements) {}

module.exports = {
  createPerson,
  findByName,
  deleteById,
  updatePersonById,
  getPersonFullDetails,
};
